#############
# $Id: md5.pm 12581 2011-02-11 12:26:13Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-07-10
# Last Changed On: $Date: 2011-02-11 12:26:13 +0000 (Fri, 11 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-25.0/lib/npg_pipeline/archive/file/generation/md5.pm $

package npg_pipeline::archive::file::generation::md5;
use Moose;
use MooseX::InsideOut;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12581 $ =~ /(\d+)/mxs; $r; };
use Cwd;
use File::Slurp;

extends q{npg_pipeline::archive::file::generation};

Readonly::Scalar our $PIPELINE_MD5_WRAPPER => q{npg_pipeline_create_md5};
Readonly::Scalar our $SLOTS_PER_JOB        => 4;


=head1 NAME

npg_pipeline::archive::file::generation::md5

=head1 VERSION

$LastChangedRevision: 12581 $

=head1 SYNOPSIS

  my $oAfgmd5 = npg_pipeline::archive::file::generation::md5->new({
    run_folder => $sRunFolder,
    md5_file_name => $sMd5FileName,
    filenames => [ qw{
      1234_1.fastq
      1234_1.fastqcheck
    } ], # example filenames
    archive_path => $sArchivePath # there must be a way of the object determining this
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating md5 files
which contain the md5 values for the archival files for the lane this represents.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the md5 files, returning an array of job_ids.

  my @job_ids = $oAfgmd5->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=cut

sub generate {
  my ($self, $arg_refs) = @_;

  my $id_run = $self->id_run();
  $self->log(qq{Creating Jobs to create md5 files for $id_run});

  my @job_ids;

  my $run = $self->run();
  $arg_refs->{run} = $run;
  my @positions = $self->positions($arg_refs);

  my $href = $self->lanes_by_split_nonsplit( @positions );

  my %lane_groups = (
    split_nc_lanes      => $href->{split_nc_lanes},
    non_split_nc_lanes  => $href->{non_split_nc_lanes},
  );

  foreach my $key ( sort keys %lane_groups ) {
    $arg_refs->{lane_group} = $lane_groups{$key};
    $arg_refs->{group_type} = $key;
    if ( scalar @{ $lane_groups{$key} } ) {
      push @job_ids, $self->submit_bsub_command( $self->_generate_bsub_command( $arg_refs ) );
    }
  }

  return @job_ids;
}

=head2 filenames

arrayref of filenames in the archive directory that we want to try to ensure we have the md5 of

=cut

has q{filenames} => (
  isa => q{ArrayRef},
  is  => q{rw},
  documentation => q{Array of files to be md5sum'd},
  traits  => [ q{Array} ],
  default => sub { [] },
  handles => {
    all_filenames    => q{elements},
    add_filename     => q{push},
    has_no_filenames => q{is_empty},
  },
);

=head2 md5_file_name

string of the name that the md5 file for this lane should be

=cut

has q{md5_file_name} => (
  isa => q{Str},
  is  => q{ro},
  documentation => q{file to put md5 sums into},
  predicate => q{has_md5_file_name},
);

=head2 generate_md5s

This method changes into the archive directory, runs through the filenames and tries to ensure that an md5
sum is recorded in the file (name in md5_file_name).

If there is no file present then it won't bother with it (rather than croak).

If there is a file which is one of the files, with .md5 appended, then it will get the md5sum out of that.

Otherwise, it will take the file and run md5sum on it.

If md5_file_name and/or filenames are not provided on construction, then this will log a message, but
otherwise will not error. It won't create an md5 file either.

  $oAfgmd5->generate_md5s();

=cut

sub generate_md5s {
  my ( $self ) = @_;

  if ( ! $self->has_md5_file_name() || $self->has_no_filenames() ) {
    $self->log( q{No md5 filename or files to be md5sum'd provided} );
    $self->log( q{This does not croak on this issue} );
    return 1;
  }

  my $archive_path = $self->archive_path();

  my $cwd = getcwd(); # to return to the cwd later
  chdir $archive_path or croak qq{unable to change to archive path $archive_path : $EVAL_ERROR};

  my $md5_file_name = $self->md5_file_name();

  if ( -e $md5_file_name ) {
    $self->log( $md5_file_name . q{ already exists, not processing further} );
    chdir $cwd;
    return 1;
  }

  qx{touch $md5_file_name};

  foreach my $filename ( $self->all_filenames() ) {

    # if the lane is split, we don't want to bother recording the full md5's since they won't be archived
    if ( $self->_is_split_lane()
           &&
         ( $filename =~ /fastq\z/xms )
           &&
         ( $filename !~ /human/xms )
       ) {
      $self->log( q{This lane is split nonconsented, and so there is no need to record the md5 for } . $filename );
      next;
    }

    my $md5 = q{};

    # if there is a filename.md5, then retrieve the md5sum from this
    if ( -e $filename . q{.md5} ) {

      $self->log( $filename . q{ has an md5 file associated with it, using that} );
      my $content = read_file( $filename . q{.md5} );
      $self->log( $content );
      ($md5, ) = split /[ ]/xms, $content;

    # if there is no file with that filename, then don't worry about it
    } elsif ( ! -e $filename ) {

      $self->log( $filename . q{ is not available, not concerned} );

    # otherwise, we need to md5sum the file
    } else {
      my $prefix = $ENV{TEST_BIN} ||  q{};
      if ( $prefix ) {
        $prefix = $cwd . q{/} . $prefix;
      }
      my $md5cmd = $prefix . $self->filesystem_locations_conf()->{user_bin} . q{/md5sum } . $filename;
      $self->log( $md5cmd );
      my $output = qx{$md5cmd};
      $self->log( $output );
      ($md5, ) = split /[ ]/xms, $output;
    }

    # if an md5sum has been obtained, write it into the md5 file - we'll include the archive path as somethings
    # downstream reading this might want the full path
    if ( $md5 ) {
      my $write_it_cmd = qq{echo '$md5 *$archive_path/$filename' >> $md5_file_name};
      my $output = qx{$write_it_cmd};
      if ( $CHILD_ERROR ) {
        croak qq{$write_it_cmd failed: $output\n\t$EVAL_ERROR};
      }
    }

  }

  $self->log( read_file( $md5_file_name ) );

  chdir $cwd;
  return 1;
}

has q{_is_split_lane} => (
  isa => q{Bool},
  is => q{ro},
  writer => q{_set_is_split_lane},
  lazy_build => 1,
);

sub _build__is_split_lane {
  my ( $self ) = @_;

  my ($lane) = $self->md5_file_name() =~ /\d+_(\d).md5/ixms;

  my $split_info = $self->lanes_by_split_nonsplit( $lane );

  return ( scalar @{ $split_info->{split_nc_lanes} } ? 1 : 0 );
}

sub _create_filenames {
  my ( $self, $arg_refs ) = @_;
  my $file_name = $self->lsb_jobindex();

  my $pr = $self->is_paired_read();
  my $ir = $self->is_indexed();

  my $group_type = $arg_refs->{group_type};

  # paired reads or indexed reads have a read number/letter in the readnames
  if ( $pr || $ir ) {

    $self->add_filename( $self->_get_filename_split( {
      lane => $file_name,
      type => q{fastq},
      end => 1,
      group_type => $group_type,
    } ) );

    $self->add_filename( $self->_get_filename_split( {
      lane => $file_name,
      type => q{fastqcheck},
      end => 1,
      group_type => $group_type,
    } ) );

    # if paired read, then add in the read 2 files for md5sum'ing
    if ( $pr ) {
      $self->add_filename(  $self->_get_filename_split( {
        lane => $file_name,
        type => q{fastq},
        end => 2,
        group_type => $group_type,
      } ) );

      $self->add_filename(  $self->_get_filename_split( {
        lane => $file_name,
        type => q{fastqcheck},
        end => 2,
        group_type => $group_type,
      } ) );

    }

    # if indexed read, then add in the tag files for md5sum'ing
    if ( $ir ) {

      $self->add_filename( $self->_get_filename_split( {
        lane => $file_name,
        type => q{fastq},
        end => q{t},
        group_type => $group_type,
      } ) );

      $self->add_filename( $self->_get_filename_split( {
        lane => $file_name,
        type => q{fastqcheck},
        end => q{t},
        group_type => $group_type,
      } ) );

    }

  } else {

    $self->add_filename( $self->_get_filename_split( {
      lane => $file_name,
      type => q{fastq},
      group_type => $group_type,
    } ) );

    $self->add_filename( $self->_get_filename_split( {
      lane => $file_name,
      type => q{fastqcheck},
      group_type => $group_type,
    } ) );

  }

  return 1;

}

###############
# responsible for generating the bsub command to be executed
sub _generate_bsub_command {
  my ( $self, $arg_refs ) = @_;

  my $lane_group = $arg_refs->{lane_group};
  my $group_type = $arg_refs->{group_type};
  my $file_name = $self->lsb_jobindex();
  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();
  my $user_bin = $self->default_usr_bin();
  my $id_run = $self->id_run();
  my $run = $arg_refs->{run};

  my $outfile = $self->make_log_dir( $self->archive_path() ) . q{/} . $self->bsub_output_file();

  my $job_name = q{md5_} . $id_run . q{_} . $group_type . q{_} . $timestamp;
  my $replacement = $job_name . q{.%I.%J};
  $job_name .= $self->create_array_string( @{ $lane_group } );

  $outfile =~ s/substitute_this/$replacement/xms;

  my $md5_file = $self->md5_file_constructs();
  $md5_file =~ s/substitute_this/$file_name/xms;

  if ( $self->has_no_filenames() ) {
    $self->_create_filenames( $arg_refs );
  }

  my $options = $self->attributes_as_command_options( {
    excluded_attributes => [ qw(
      log_file log_file_name log_file_path subpath casava_pipeline illumina_pipeline timestamp
      bustard_home gerald_config_prefix runtype script_name time md5_file_name bsub_output_file
      tag_qseqs_path
      ) ],
  } );

  my $md5_creator_script = $self->md5_creator_script();
  my $job_sub = q{bsub -q } . $self->lsf_queue() . q{ };

  $job_sub .=  ( $self->fs_resource_string( {
    counter_slots_per_job => $SLOTS_PER_JOB,
  } ) ) . q{ };

  $job_sub .= qq{$required_job_completion -J $job_name -o $outfile '} . $self->internal_script_bin() . qq{/$PIPELINE_MD5_WRAPPER  $options --md5_file_name $md5_file'};

  if ( $self->verbose() ) {
    $self->log( $job_sub );
  }

  return $job_sub;
}

sub _get_filename_split {
  my ( $self, $arg_refs ) = @_;

  my $lane = $arg_refs->{lane};
  my $type = $arg_refs->{type};
  my $end  = $arg_refs->{end};
  my $group_type = $arg_refs->{group_type};

  my $middle_file_name = $lane;

  if ( $end ) {
    $middle_file_name .= q{_} . $end;
  }

  my $middle_file_name1 = $middle_file_name;

  if ( ( $group_type eq q{split_nc_lanes} ) && ( $type ne q{fastqcheck} ) ) {

      $middle_file_name1 .= q{_nonhuman};

  }

  my $file_params = $self->_get_filename( $middle_file_name1, $type );

  if ( ( $group_type eq q{split_nc_lanes} ) && ( $type ne q{fastqcheck} ) ) {

      my $middle_file_name2 = $middle_file_name . q{_human};

      $file_params .= q{ }.$self->_get_filename( $middle_file_name2, $type );
  }

  return $file_params;
}

sub _get_filename {
  my ( $self, $lane_end_split, $type ) = @_;

  my $construct_method_name = $type . q{_file_constructs};
  my $file = $self->$construct_method_name();
  $file =~ s/substitute_this/$lane_end_split/xms;

  return $file;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__


=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
