#############
# $Id: bustard_lsf_reqs.pm 12581 2011-02-11 12:26:13Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-11-02
# Last Changed On: $Date: 2011-02-11 12:26:13 +0000 (Fri, 11 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-25.0/lib/npg_pipeline/roles/business/bustard_lsf_reqs.pm $

package npg_pipeline::roles::business::bustard_lsf_reqs;
use Moose::Role;
use Carp;
use File::Slurp;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12581 $ =~ /(\d+)/mxs; $r; };

##############
# defaults
# these defaults are not in a config file, as they should almost never need changing, and are related to the bustard/gerald process
Readonly::Scalar our $MAKE_EXE => '/make';                        # the latest -- will prepend internal_script_bin when called via the method
Readonly::Scalar our $POST_QSEQ_EXE => '/npg_pipeline_post_qseq'; # will prepend internal_script_bin when called via the method
Readonly::Scalar our $BUSTARD_HOME => 'Data/Intensities';   # runfolder subfolder where bustard data goes
Readonly::Scalar our $BUSTARD_PREFIX => 'bustard-output';   # will have timestamp and .txt appended
Readonly::Scalar our $GERALD_PREFIX  => 'gerald-output';    # will have timestamp and .txt appended
Readonly::Scalar our $GERALD_CONFIG  => 'gerald-config';    # will have timestamp and .txt appended
Readonly::Scalar our $RUNTYPE      => 'RTA';                # phony username for analysis directories
Readonly::Scalar our $MEM_REQ => 13_800;                    # total MB used by a make
Readonly::Scalar our $DUMP_SIZE => 99_999_999;              # max dump allowed (-C parameter of bsub)


=head1 NAME

npg_pipeline::roles::business::bustard_lsf_reqs

=head1 VERSION

$LastChangedRevision: 12581 $

=head1 SYNOPSIS

  package MyPackage;
  use Moose;
  ...
  with qw{npg_pipeline::roles::business::bustard_lsf_reqs};

=head1 DESCRIPTION

This role is designed to be able to apply all the bustard variables, including lsf requirements, for internal running of the pipeline.
It does require an internal_script_bin method to be available.

=head1 SUBROUTINES/METHODS

=head2 illumina_pipeline

string containing path to the pipeline to be used, can be set on construction if you wish to pass in a different pipeline location

  my $sIlluminaPipeline = $class->illumina_pipeline();

=cut

has q{illumina_pipeline} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => q{enables the user to set the illumina analysis pipeline to be used on the command line},);
sub _build_illumina_pipeline {
  my ($self) = @_;

  my $src = $self->filesystem_locations_conf()->{src};
  my $olb = $self->illumina_pipeline_conf()->{olb};

  return $src . q{/} . $olb;
}

=head2 casava_pipeline

string containing path to the casava pipeline to be used, can be set on construction if you wish to pass in a different pipeline location

  my $sCasavaPipeline = $class->casava_pipeline();

=cut

has q{casava_pipeline} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => q{enables the user to set the CASAVA analysis pipeline to be used on the command line},);
sub _build_casava_pipeline {
  my ($self) = @_;
  my $src = $self->filesystem_locations_conf()->{src};
  my $casava = $self->illumina_pipeline_conf()->{casava};

  return $src . q{/} . $casava;
}

=head2 boost_lib

string containing the boost library name.

  my $sBoostLib = $class->boost_lib();

=cut

has q{boost_lib} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => q{enables the user to set the boost lib to be used on the command line},);
sub _build_boost_lib {
  my ($self) = @_;
  my $src = $self->filesystem_locations_conf()->{src};
  my $boost = $self->illumina_pipeline_conf()->{boost};

  return $src . q{/} . $boost;
}

=head2 make_executable

string containing the make executable.

  my $sMakeExecutable = $class->make_executable();

=cut

has q{make_executable} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => qq{enables the user to set the make executable to be used on the command line - DEFAULT $MAKE_EXE},);
sub _build_make_executable {
  my ($self) = @_;
  return $self->internal_script_bin() . $MAKE_EXE;
}

=head2 post_qseq_executable

string containing the post_qseq executable.

  my $sPostQSEQExecutable = $class->post_qseq_executable();

=cut

has q{post_qseq_executable} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => qq{enables the user to set the post_qseq executable to be used on the command line - DEFAULT $POST_QSEQ_EXE},);
sub _build_post_qseq_executable {
  my ($self) = @_;
  return $self->internal_script_bin() . $POST_QSEQ_EXE;
}

=head2 bustard_executable

string containing the bustard executable.

  my $sBustardExecutable = $class->bustard_executable();

=cut

has q{bustard_executable} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => q{enables the user to set the bustard executable to be used on the command line},);
sub _build_bustard_executable {
  my ($self) = @_;
  return $self->illumina_pipeline_conf()->{bustard_exe};
}

=head2 gerald_executable

string containing the gerald executable.

  my $sGeraldExecutable = $class->gerald_executable();

=cut

has q{gerald_executable} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => q{enables the user to set the gerald executable to be used on the command line},);
sub _build_gerald_executable {
  my ($self) = @_;
  return $self->illumina_pipeline_conf()->{gerald_exe};
}

=head2 bustard_home

string containing the bustard home dir.

  my $sBustardHome = $class->bustard_home();

=cut

has q{bustard_home} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => qq{enables the user to set the bustard home dir - DEFAULT $BUSTARD_HOME},);
sub _build_bustard_home {
  my ($self) = @_;
  return $BUSTARD_HOME;
}

=head2 bustard_output_prefix

string containing the bustard output file prefix.

  my $sBustardFilePrefix = $class->bustard_output_prefix();

=cut

has q{bustard_output_prefix} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => qq{enables the user to set the bustard output file prefix - DEFAULT $BUSTARD_PREFIX},);
sub _build_bustard_output_prefix {
  my ($self) = @_;
  return $BUSTARD_PREFIX;
}

=head2 gerald_output_prefix

string containing the gerald output file prefix.

  my $sGeraldFilePrefix = $class->gerald_output_prefix();

=cut

has q{gerald_output_prefix} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => qq{enables the user to set the gerald output file prefix - DEFAULT $GERALD_PREFIX},);
sub _build_gerald_output_prefix {
  my ($self) = @_;
  return $GERALD_PREFIX;
}

=head2 gerald_config_prefix

string containing the gerald config file prefix.

  my $sGeraldConfigPrefix = $class->gerald_config_prefix();

=cut

has q{gerald_config_prefix} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => qq{enables the user to set the gerald config file prefix - DEFAULT $GERALD_CONFIG},);
sub _build_gerald_config_prefix {
  my ($self) = @_;
  return $GERALD_CONFIG;
}

=head2 runtype

string containing the runtype. Mostly to be used as a phony username for directories

  my $sRuntype = $class->runtype();

=cut

has q{runtype} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => qq{enables the user to set the runtype - DEFAULT $RUNTYPE},);
sub _build_runtype {
  my ($self) = @_;
  return $RUNTYPE;
}

=head2 preferred_control_lane

integer containing the preferred control lane.

  my $iPreferredControlLane = $class->preferred_control_lane();

=cut

has q{preferred_control_lane} => (isa => q{Int}, is => q{ro}, lazy_build => 1,
                      documentation => q{enables the user to set the preferred control lane},);
sub _build_preferred_control_lane {
  my ($self) = @_;
  return $self->general_values_conf()->{preferred_control_lane};
}

=head2 memory_requirement

integer containing the number of memory required to be used in an lsf command.

  my $iMemoryRequirement = $class->memory_requirement();

=cut

has q{memory_requirement} => (isa => q{Int}, is => q{ro}, lazy_build => 1,
                      documentation => qq{enables the user to set the memory to be required in an lsf command - DEFAULT $MEM_REQ},);
sub _build_memory_requirement {
  my ($self) = @_;
  return $MEM_REQ;
}

=head2 dump_size

integer containing the dump size to be used in an lsf command.

  my $iDumpSize = $class->dump();

=cut

has q{dump_size} => (isa => q{Int}, is => q{ro}, lazy_build => 1,
                      documentation => qq{enables the user to set the dump size to be used in an lsf command - DEFAULT $DUMP_SIZE},);
sub _build_dump_size {
  my ($self) = @_;
  return $DUMP_SIZE;
}

=head2 find_bustard

finds the path to the bustard directory in the following preferential order

1) If it has been provided as a command option as 'bustard_path'
2) If it is present in a conf file (in the runfolder directory) with the same timestamp as this run through
3) The latest updated bustard directory, and then store this in mentioned conf file

It then returns one of these

  my $sBustardPath = $class->find_bustard();

Note: Unless it has been specified upfront (because bustard itself isn't getting run for example), then there
is still a moment in time that another bustard directory could get updated before the one required is found. As
running bustard does not return the bustard path, this cannot be helped.

Note: this will croak with a run_time error if there is no runfolder_path method available

=cut

# ----------------------------------------------------------------------
# Find the latest-created Bustard directory in the runfolder --
# presumably the one we're supposed to process.

sub find_bustard {

  my ($self) = @_;

  if ($self->can(q{has_bustard_path}) && $self->has_bustard_path()) {
    $self->log( q{bustard_path already populated with } . $self->bustard_path() );
    return $self->bustard_path();
  }

  my $run_folder    = $self->runfolder_path();               # fully qualified name
  my $bustard_home  = "$run_folder/" . $self->bustard_home();
  my $runtype = $self->runtype();

  # since this clearly wasn't present in the object, see if it is in the conf_file for this run
  # which should be in the runfolder if it has been generated
  my $file_config = $self->run_conf();

  # if the bustard_path is in the conf file, use it
  if ( $file_config->{bustard_path} ) {
    $self->log( q{retrieved bustard_path from conf file} );
    return $file_config->{bustard_path};
  }

  # here is still a potential problem, since if the another bustard dir gets updated prior to this being done,
  # it will still retrieve the wrong one. However, we hope that the only time it needs retrieving from the fs
  # will be mere moments after it has been created
  my @bustards = qx[ls -1td $bustard_home/Bustard*$runtype*];
  croak "no bustard_$runtype directories in $bustard_home" if @bustards == 0;
  my $bustard_dir = $bustards[0];                            # fully qualified name
  chomp $bustard_dir;

  # since we have had to retrieve this from the fs already, now store in the conf file
  $self->add_to_run_conf( {
    bustard_path => $bustard_dir,
  } );

  return $bustard_dir;

}

=head2 pick_control_lane

Look for an appropriate control lane. Choose
$self->preferred_control_lane() if it's a control, otherwise pick
the first control lane we come to. Chosen lane must have a reference
defined for it. Return null string if no suitable control lane
found.  If the no_control_lane flag has been set, then return as
though no control lane has been found

=cut

# ----------------------------------------------------------------------

sub pick_control_lane {

  my ($self) = @_;

  my $control_lane = q{};

  # check to see if no_control_lane flag specified was specified
  if ($self->no_control_lane()) {
    $self->log(q{'No Control Lane' specified.});
    return $control_lane;
  }

  if ($self->has_control_lane_position()) {
    return $self->control_lane_position();
  }

  my $run = $self->run();

  foreach my $run_lane (@{$run->run_lanes()}) {

    my $control  = $run_lane->control();

    if ($control) {

      my $position = $run_lane->position();
      my $ref = $control->reference_sequence();

      if ($ref) {
        if ( ! $control_lane || ($position == $self->preferred_control_lane())) {
          $control_lane = $position;
        }
      }

    }

  }

  if ( ! $control_lane) {
    my $run_name = $run->name();
    $self->log("$run_name WARNING: run has no suitable control lane");
  }

  return $control_lane;

}

=head2 phasing

The phasing to be used for bustard.py
Defaults to the value in the illuming_pipeline config file for the type of analysis done (lane or control based)
If you are user defining this on the command line, you need to add prephasing

=head2 phasing_value

The correct way to get the phasing value to use on the bustard.py command line

=cut

has q{phasing} => (
  isa => q{Str},
  is => q{ro},
  predicate => q{has_phasing},
  documentation => q{Override the phasing parameter to be passed to bustard.py - requires you to add a prephasing value as well},
);

sub phasing_value {
  my ( $self, $control_lane ) = @_;

  if ( $self->has_phasing() ) {
    if ( ! $self->has_prephasing() ) {
      croak q{User defined phasing given, but no user defined prephasing given};
    }
    return $self->phasing();
  }

  if ( $control_lane ) {
    my $phasing = $self->illumina_pipeline_conf()->{control_based_phasing};
    $phasing =~ s/CONTROL_LANE/$control_lane/xms;
    return $phasing;
  }

  return $self->illumina_pipeline_conf()->{lane_based_phasing};
}


=head2 prephasing

The prephasing to be used for bustard.py
Defaults to not being given, so you need to give it if you are using a user defined phasing

=cut

has q{prephasing} => (
  isa => q{Str},
  is => q{ro},
  predicate => q{has_prephasing},
  documentation => q{Override the phasing parameter to be passed to bustard.py},
);

=head2 matrix

The crosstalk matrix to be used for bustard.py
Defaults to the value in the illuming_pipeline config file for the type of analysis done (lane or control based)

=head2 matrix_value

The correct way to obtain the value to be used for the matrix on the bustard.py command line

=cut

has q{matrix} => (
  isa => q{Str},
  is => q{ro},
  predicate => q{has_matrix},
  documentation => q{Override the crosstalk matrix to be passed to bustard.py},
);

sub matrix_value {
  my ( $self, $control_lane ) = @_;

  if ( $self->has_matrix() ) {
    return $self->matrix();
  }

  if ( $control_lane ) {
    my $crosstalk_matrix = $self->illumina_pipeline_conf()->{control_based_matrix};
    $crosstalk_matrix =~ s/CONTROL_LANE/$control_lane/xms;
    return $crosstalk_matrix;
  }

  return $self->illumina_pipeline_conf()->{lane_based_matrix};
}

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
