#############
# $Id: flag_options.pm 12581 2011-02-11 12:26:13Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-11-30
# Last Changed On: $Date: 2011-02-11 12:26:13 +0000 (Fri, 11 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-25.0/lib/npg_pipeline/roles/business/flag_options.pm $

package npg_pipeline::roles::business::flag_options;
use Moose::Role;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12581 $ =~ /(\d+)/mxs; $r; };

=head1 NAME

npg_pipeline::roles::business::flag_options

=head1 VERSION

$LastChangedRevision: 12581 $

=head1 SYNOPSIS

  package MyPackage;
  use Moose;
  ...
  with qw{npg_pipeline::roles::business::flag_options};

=head1 DESCRIPTION

This role gives some boolean flag options which should be set on construction (or via the command line if using MooseX::Getopt)
so that you can turn off global features/functions without having to necessarily specify them.

  --no_status_updates
  --no_folder_moves
  --no_control_lane

These would globally stop anything being done should functions be requested which do these (either directly, or by job submission)

=head1 SUBROUTINES/METHODS

=head2 no_bsub

Disable submitting any jobs to bsub, so the pipeline can be run, and all cmds logged

=cut

has q{no_bsub} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Turn off submitting any jobs to lsf, just logging them instead},
);

=head2 no_status_updates

Universally disables status_updates

=cut

has q{no_status_updates} => (isa => q{Bool}, is => q{ro},
  documentation => q{Turn off updating any tracking statuses},);

=head2 no_folder_moves

Universally disables anything that would move a folder

=cut

has q{no_folder_moves} => (isa => q{Bool}, is => q{ro},
  documentation => q{Turn off moving any folders},);

=head2 no_summary_link

Do not create a summary link

=cut

has q{no_summary_link} => (isa => q{Bool}, is => q{ro}),
  documentation => q{Turn off creating a Latest_Summary link},;

=head2 no_post_qseq

Do not launch any post_qseq scripts

=cut

has q{no_post_qseq} => (isa => q{Bool}, is => q{ro},
  documentation => q{Turn off launching any post_qseq (archival file creation) scripts},);

=head2 no_control_lane

Act as though there is no control lane for recalibration purposes

=cut

has q{no_control_lane} => (isa => q{Bool}, is => q{ro}, lazy_build => 1,
  documentation => q{If you don't want any control lane recalibration, act as though there is no control lane},);

sub _build_no_control_lane {
  my ( $self ) = @_;
  return $self->no_recalibration();
}

=head2 no_recalibration

Act as though no recalibration should be done, regardless of the presence of a control lane or not
This will automatically set no_control_lane to true.

=cut

has q{no_recalibration} => (isa => q{Bool}, is => q{ro},
  documentation => q{Turn off doing any recalibration, regardless of control lane status}, );

=head2 no_pb_cal

Do not launch the pb_cal script - only for analyse_rta (bustard) pipeline

=cut

has q{no_pb_cal} => (isa => q{Bool}, is => q{ro},
  documentation => q{Turn off running the pb_cal script}, );

=head2 no_bam_generation

Skip creating bam files

=cut

has q{no_bam_generation} => (isa => q{Bool}, is => q{ro},
  documentation => q{Turn off creating bam files},);

=head2 no_split_fastq_by_tag

Skip splitting fastq by tag for multiplex run

=cut

has q{no_split_fastq_by_tag} => (isa => q{Bool}, is => q{ro},
  documentation => q{Turn off splitting fastq by tag for multiplex run},);

=head2 no_eland

Don't run eland on anything. There is an option no_eland, which needs to be set,
but the return value is still false if no_control_lane is not set (i.e you need both)

=cut

sub no_eland {
  my ($self) = @_;
  if ($self->no_eland_running() && $self->no_control_lane()) {
    return 1;
  }
  return 0;
}

has q{no_eland_running} => (isa => q{Bool}, is => q{ro}, init_arg => q{no_eland},
  documentation => q{Do not run eland. Only applies if no_control_lane also set},);

=head2 no_spider

flag option to not spider out at all for webservice data

=cut

has q{no_spider} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Turn off spidering and caching webservice data},
);

=head2 no_cached_webservice_data

flag option to not used cached webservice data for this

=cut

has q{no_cached_webservice_data} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Turn off using any cached webservice data},
);


=head2 no_munge_summary

flag option to request no summary munging (which is only for multiplexed lanes which have have GERALD run)

=cut

has q{no_munge_summary} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Do not run munge summary},
);

=head2 no_fix_config_files

flag option to request that config files are not checked and fixed

=cut

has q{no_fix_config_files} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{request that config files are not checked and fixed (where fixing is appropriate)},
);

=head2 no_array_cpu_limit

flag option to allow job arrays to flood, if able, the farm

=cut

has q{no_array_cpu_limit} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Allow job arrays to keep launching if cpus available},
);

=head2 array_cpu_limit

set the most number of cpus which each job array can use at a time, applied only if no_array_cpu_limit not set

=cut

has q{array_cpu_limit} => (
  isa => q{Int},
  is => q{ro},
  documentation => q{Set the most number of CPUs that a Job array can use at a time},
  lazy_build => 1,
);

sub _build_array_cpu_limit {
  my ( $self ) = @_;
  return $self->general_values_conf()->{array_cpu_limit};
}
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
